/* Copyright 2013-2017 Sathya Laufer
 *
 * libhomegear-base is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libhomegear-base is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libhomegear-base.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU Lesser General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
*/

#include "DevicePacketResponse.h"
#include "../BaseLib.h"

namespace BaseLib
{
namespace DeviceDescription
{

DevicePacketResponse::DevicePacketResponse(BaseLib::SharedObjects* baseLib)
{
    _bl = baseLib;
}

DevicePacketResponse::DevicePacketResponse(BaseLib::SharedObjects* baseLib, xml_node<>* node) : DevicePacketResponse(baseLib)
{
    for(xml_attribute<>* attr = node->first_attribute(); attr; attr = attr->next_attribute())
    {
        _bl->out.printWarning("Warning: Unknown attribute for \"packet\\responses\\response\": " + std::string(attr->name()));
    }
    for(xml_node<>* subNode = node->first_node(); subNode; subNode = subNode->next_sibling())
    {
        std::string nodeName(subNode->name());
        std::string value(subNode->value());
        if(nodeName == "responseId") responseId = value;
        else if(nodeName == "conditionOperator")
        {
            HelperFunctions::toLower(HelperFunctions::trim(value));
            if(value == "e" || value == "eq") conditionOperator = ConditionOperator::Enum::e;
            else if(value == "g") conditionOperator = ConditionOperator::Enum::g;
            else if(value == "l") conditionOperator = ConditionOperator::Enum::l;
            else if(value == "ge") conditionOperator = ConditionOperator::Enum::ge;
            else if(value == "le") conditionOperator = ConditionOperator::Enum::le;
            else _bl->out.printWarning("Warning: Unknown element in \"conditionOperator\": " + value);
        }
        else if(nodeName == "conditionParameterId") conditionParameterId = value;
        else if(nodeName == "conditionChannel") conditionChannel = Math::getNumber(value);
        else if(nodeName == "conditionValue") conditionValue = Math::getNumber(value);
        else _bl->out.printWarning("Warning: Unknown node in \"packet\\responses\\response\": " + nodeName);
    }
}

bool DevicePacketResponse::checkCondition(int32_t lhs)
{
    switch(conditionOperator)
    {
        case ConditionOperator::Enum::e:
            return lhs == conditionValue;
        case ConditionOperator::Enum::g:
            return lhs > conditionValue;
        case ConditionOperator::Enum::l:
            return lhs < conditionValue;
        case ConditionOperator::Enum::ge:
            return lhs >= conditionValue;
        case ConditionOperator::Enum::le:
            return lhs <= conditionValue;
        default:
            return false;
    }
}

}
}
